import logging

from flask import Blueprint
from flask import abort
from flask import flash
from flask import redirect
from flask import render_template
from flask import request
from flask import url_for
from flask_login import login_user as fl_login_user
from flask_login import logout_user as fl_logout_user
from pydantic import BaseModel
from pydantic import ValidationError
from sqlalchemy import select

from htmx_contact import Session
from htmx_contact.models import User

bp = Blueprint("user", __name__, url_prefix="/user")

logger = logging.getLogger(__name__)


class LoginValidator(BaseModel):
    """Used to validate user login"""

    email: str
    password: str


@bp.route("/login", methods=["GET", "POST"])
def user_login():
    if request.method == "POST":
        try:
            data = LoginValidator.model_validate(dict(request.form))
        except ValidationError:
            logger.warning("User input failed validation")
            abort(422)

        logger.info(f"Received login request from {data.email}")
        data = LoginValidator.model_validate(dict(request.form))
        with Session() as session:
            select_user_stmt = select(User).where(User.primary_email == data.email)
            user = session.scalar(select_user_stmt)
            if user is not None and user.check_password(data.password):
                # Login and redirect from where they came from
                fl_login_user(user=user)
                flash("Welcome back {}.".format(user.username))
                return redirect(request.args.get("next") or url_for("main.contacts"))
            # User was None or password was incorrect
            flash("Incorrect username or password")
        return render_template("login.html")
    else:
        return render_template("login.html")


@bp.route("/logout", methods=["GET"])
def user_logout():
    fl_logout_user()
    """Logs a user out of thier session"""
    return redirect(url_for('main.contacts'))


@bp.route("/sign-up", methods=["GET"])
def user_sign_up():
    """Renders the signup form to the user"""
    return render_template("sign-up.html")


@bp.route("/", methods=["POST"])
def create_user():
    """Creates a new user"""
    # create the user.
    # Add message flash
    return redirect(url_for("user.user_login"))
