import re

from pydantic import BaseModel
from pydantic import Field
from pydantic import field_validator

from htmx_contact.models import PHONE_REGEX

EMAIL_REGEX = r'^[\w\-\.]+@([\w-]+\.)+[\w-]{2,}$'


def repack_validation_errors(errors):
    repackaged_errors = {error.get('loc')[0]: error.get('msg') for error in errors}
    return repackaged_errors


class NewContactValidator(BaseModel):
    first_name: str = Field(min_length=2, max_length=30)
    last_name: str = Field(min_length=2, max_length=30)
    phone: str = Field(max_length=30)
    email: str = Field(max_length=120)

    # Field Validators used to prevent regex patterns returned in error message
    @field_validator('phone')
    def must_match_phone_pattern(value):
        if not bool(re.match(PHONE_REGEX, value)):
            raise ValueError("not a valid phone number. Please review and resubmit.")
        return value

    @field_validator('email')
    def must_match_email_pattern(value):
        if not bool(re.match(EMAIL_REGEX, value)):
            raise ValueError("not a valid email. Please review and resubmit.")
        return value
